import secrets
import string

from flask import Blueprint, request, redirect, render_template
from flask_login import current_user, login_user, logout_user, LoginManager
from typing import Any, Dict, List, Optional


bp = Blueprint("auth", __name__, url_prefix="/auth")


ACCESS_TOKEN = "".join(secrets.choice(string.ascii_lowercase) for i in range(64))


ALLOWED_PATHS = [
    "/favicon.ico",
    "/static/jon.css"
]


# A poor man's replacement for memory-backed session solution.
# We keep exactly one User (and the corresponding UserData) in
# memory and use that to store session data.
class UserData:
    location: Optional[Dict[str, Any]]
    orders: List[Dict[str, Any]]

    def __init__(self):
        self.location = None
        self.orders = []


class User:
    is_authenticated: bool
    is_active: bool
    is_anonymous: bool

    data: UserData

    def __init__(self):
        self.is_authenticated = True
        self.is_active = True
        self.is_anonymous = False

        self.data = UserData()

    def get_id(self) -> str:
        return ""


def init_app(app):
    login_manager = LoginManager(app)
    the_one_and_only_user = User()

    @login_manager.user_loader
    def load_user(user_id: str) -> User:
        assert user_id == ""
        return the_one_and_only_user

    # This function denies every request until `auth.ACCESS_TOKEN`
    # is passed using `?token=` to authenticate the user.
    # We use this instead of @login_required because otherwise we'd have
    # to add that annotation to all routes.
    # See also: https://flask-login.readthedocs.io/en/latest/#flask_login.login_required
    @app.before_request
    def before_request():
        if "token" in request.args:
            if request.args["token"] == ACCESS_TOKEN:
                login_user(the_one_and_only_user)
            # Reload the page without query parameters
            return redirect(request.path)

        # Never deny any paths in `ALLOWED_PATHS`
        if request.path in ALLOWED_PATHS:
            return

        if not current_user.is_authenticated:
            return render_template("auth/denied.html"), 403


@bp.get("/logout")
def logout():
    logout_user()
    return redirect("/")
